@@ -5,7 +5,9 @@ format, and print it on stdout. See the manual for more details.
 
 module Hledger.Cli.Convert where
 import Control.Monad (when, guard, liftM)
+import Data.List
 import Data.Maybe
+import Data.Ord
 import Data.Time.Format (parseTime)
 import Safe
 import System.Directory (doesFileExist)
@@ -14,7 +16,7 @@ import System.FilePath (takeBaseName, replaceExtension)
 import System.IO (stderr)
 import System.Locale (defaultTimeLocale)
 import Test.HUnit
-import Text.CSV (parseCSV, parseCSVFromFile, printCSV, CSV)
+import Text.CSV (parseCSV, parseCSVFromFile, CSV)
 import Text.ParserCombinators.Parsec
 import Text.Printf (hPrintf)
 
@@ -37,8 +39,8 @@ data CsvRules = CsvRules {
       codeField :: Maybe FieldPosition,
       descriptionField :: [FormatString],
       amountField :: Maybe FieldPosition,
-      inField :: Maybe FieldPosition,
-      outField :: Maybe FieldPosition,
+      amountInField :: Maybe FieldPosition,
+      amountOutField :: Maybe FieldPosition,
       currencyField :: Maybe FieldPosition,
       baseCurrency :: Maybe String,
       accountField :: Maybe FieldPosition,
@@ -55,8 +57,8 @@ nullrules = CsvRules {
       codeField=Nothing,
       descriptionField=[],
       amountField=Nothing,
-      inField=Nothing,
-      outField=Nothing,
+      amountInField=Nothing,
+      amountOutField=Nothing,
       currencyField=Nothing,
       baseCurrency=Nothing,
       accountField=Nothing,
@@ -86,11 +88,11 @@ convert opts = do
       usingStdin = csvfile == "-"
       rulesFileSpecified = isJust $ rules_file_ opts
       rulesfile = rulesFileFor opts csvfile
-  when (usingStdin && (not rulesFileSpecified)) $ error' "please use --rules to specify a rules file when converting stdin"
+  when (usingStdin && (not rulesFileSpecified)) $ error' "please use --rules-file to specify a rules file when converting stdin"
   csvparse <- parseCsv csvfile
   let records = case csvparse of
                   Left e -> error' $ show e
-                  Right rs -> reverse $ filter (/= [""]) rs
+                  Right rs -> filter (/= [""]) rs
   exists <- doesFileExist rulesfile
   if (not exists) then do
                   hPrintf stderr "creating conversion rules file %s, edit this file for better results\n" rulesfile
@@ -104,7 +106,8 @@ convert opts = do
   let requiredfields = max 2 (maxFieldIndex rules + 1)
       badrecords = take 1 $ filter ((< requiredfields).length) records
   if null badrecords
-   then mapM_ (printTxn (debug_ opts) rules) records
+   then do
+     mapM_ (putStr . show) $ sortBy (comparing tdate) $ map (transactionFromCsvRecord rules) records
    else do
      hPrintf stderr (unlines [
                       "Warning, at least one CSV record does not contain a field referenced by the"
@@ -127,8 +130,8 @@ maxFieldIndex r = maximumDef (-1) $ catMaybes [
                   ,statusField r
                   ,codeField r
                   ,amountField r
-                  ,inField r
-                  ,outField r
+                  ,amountInField r
+                  ,amountOutField r
                   ,currencyField r
                   ,accountField r
                   ,account2Field r
@@ -149,7 +152,7 @@ initialRulesFileContent =
     "date-field 0\n" ++
     "description-field 4\n" ++
     "amount-field 1\n" ++
-    "currency $\n" ++
+    "base-currency $\n" ++
     "\n" ++
     "# account-assigning rules\n" ++
     "\n" ++
@@ -166,14 +169,14 @@ initialRulesFileContent =
 validateRules :: CsvRules -> Maybe String
 validateRules rules = let
     hasAmount = isJust $ amountField rules
-    hasIn = isJust $ inField rules
-    hasOut = isJust $ outField rules
+    hasIn = isJust $ amountInField rules
+    hasOut = isJust $ amountOutField rules
   in case (hasAmount, hasIn, hasOut) of
-    (True, True, _) -> Just "Don't specify in-field when specifying amount-field"
-    (True, _, True) -> Just "Don't specify out-field when specifying amount-field"
-    (_, False, True) -> Just "Please specify in-field when specifying out-field"
-    (_, True, False) -> Just "Please specify out-field when specifying in-field"
-    (False, False, False) -> Just "Please specify either amount-field, or in-field and out-field"
+    (True, True, _) -> Just "Don't specify amount-in-field when specifying amount-field"
+    (True, _, True) -> Just "Don't specify amount-out-field when specifying amount-field"
+    (_, False, True) -> Just "Please specify amount-in-field when specifying amount-out-field"
+    (_, True, False) -> Just "Please specify amount-out-field when specifying amount-in-field"
+    (False, False, False) -> Just "Please specify either amount-field, or amount-in-field and amount-out-field"
     _ -> Nothing
 
 -- rules file parser
@@ -205,8 +208,8 @@ definitions = do
    ,codefield
    ,descriptionfield
    ,amountfield
-   ,infield
-   ,outfield
+   ,amountinfield
+   ,amountoutfield
    ,currencyfield
    ,accountfield
    ,account2field
@@ -269,17 +272,17 @@ amountfield = do
   x <- updateState (\r -> r{amountField=readMay v})
   return x
 
-infield = do
-  string "in-field"
+amountinfield = do
+  choice [string "amount-in-field", string "in-field"]
   many1 spacenonewline
   v <- restofline
-  updateState (\r -> r{inField=readMay v})
+  updateState (\r -> r{amountInField=readMay v})
 
-outfield = do
-  string "out-field"
+amountoutfield = do
+  choice [string "amount-out-field", string "out-field"]
   many1 spacenonewline
   v <- restofline
-  updateState (\r -> r{outField=readMay v})
+  updateState (\r -> r{amountOutField=readMay v})
 
 currencyfield = do
   string "currency-field"
@@ -300,7 +303,7 @@ account2field = do
   updateState (\r -> r{account2Field=readMay v})
 
 basecurrency = do
-  string "currency"
+  choice [string "base-currency", string "currency"]
   many1 spacenonewline
   v <- restofline
   updateState (\r -> r{baseCurrency=Just v})
@@ -340,11 +343,6 @@ matchreplacepattern = do
   newline
   return (matchpat,replpat)
 
-printTxn :: Bool -> CsvRules -> CsvRecord -> IO ()
-printTxn debug rules rec = do
-  when debug $ hPrintf stderr "record: %s" (printCSV [rec])
-  putStr $ show $ transactionFromCsvRecord rules rec
-
 -- csv record conversion
 formatD :: CsvRecord -> Bool -> Maybe Int -> Maybe Int -> Field -> String
 formatD record leftJustified min max f = case f of 
@@ -426,19 +424,37 @@ transactionFromCsvRecord rules fields =
 -- | Convert some date string with unknown format to YYYY/MM/DD.
 normaliseDate :: Maybe String -- ^ User-supplied date format: this should be tried in preference to all others
               -> String -> String
-normaliseDate mb_user_format s = maybe "0000/00/00" showDate $
-              firstJust $
-              (maybe id (\user_format -> (parseTime defaultTimeLocale user_format s :)) mb_user_format) $
-              [parseTime defaultTimeLocale "%Y/%m/%e" s
-               -- can't parse a month without leading 0, try adding one
-              ,parseTime defaultTimeLocale "%Y/%m/%e" (take 5 s ++ "0" ++ drop 5 s)
-              ,parseTime defaultTimeLocale "%Y-%m-%e" s
-              ,parseTime defaultTimeLocale "%Y-%m-%e" (take 5 s ++ "0" ++ drop 5 s)
-              ,parseTime defaultTimeLocale "%m/%e/%Y" s
-              ,parseTime defaultTimeLocale "%m/%e/%Y" ('0':s)
-              ,parseTime defaultTimeLocale "%m-%e-%Y" s
-              ,parseTime defaultTimeLocale "%m-%e-%Y" ('0':s)
-              ]
+normaliseDate mb_user_format s =
+    let parsewith = flip (parseTime defaultTimeLocale) s in
+    maybe (error' $ "could not parse \""++s++"\" as a date, consider adding a date-format directive or upgrading")
+          showDate $
+          firstJust $ (map parsewith $
+                       maybe [] (:[]) mb_user_format
+                       -- the - modifier requires time-1.2.0.5, released
+                       -- in 2011/5, so for now we emulate it for wider
+                       -- compatibility.  time < 1.2.0.5 also has a buggy
+                       -- %y which we don't do anything about.
+                       -- ++ [
+                       -- "%Y/%m/%d"
+                       -- ,"%Y/%-m/%-d"
+                       -- ,"%Y-%m-%d"
+                       -- ,"%Y-%-m-%-d"
+                       -- ,"%m/%d/%Y"
+                       -- ,"%-m/%-d/%Y"
+                       -- ,"%m-%d-%Y"
+                       -- ,"%-m-%-d-%Y"
+                       -- ]
+                      )
+                      ++ [
+                       parseTime defaultTimeLocale "%Y/%m/%e" s
+                      ,parseTime defaultTimeLocale "%Y-%m-%e" s
+                      ,parseTime defaultTimeLocale "%m/%e/%Y" s
+                      ,parseTime defaultTimeLocale "%m-%e-%Y" s
+                      ,parseTime defaultTimeLocale "%Y/%m/%e" (take 5 s ++ "0" ++ drop 5 s)
+                      ,parseTime defaultTimeLocale "%Y-%m-%e" (take 5 s ++ "0" ++ drop 5 s)
+                      ,parseTime defaultTimeLocale "%m/%e/%Y" ('0':s)
+                      ,parseTime defaultTimeLocale "%m-%e-%Y" ('0':s)
+                      ]
 
 -- | Apply account matching rules to a transaction description to obtain
 -- the most appropriate account and a new description.
@@ -460,13 +476,13 @@ getAmount :: CsvRules -> CsvRecord -> String
 getAmount rules fields = case amountField rules of
   Just f  -> maybe "" (atDef "" fields) $ Just f
   Nothing ->
-    case (c, d) of
+    case (i, o) of
       (x, "") -> x
       ("", x) -> "-"++x
-      _ -> ""
+      p -> error' $ "using amount-in-field and amount-out-field, found a value in both fields: "++show p
     where
-      c = maybe "" (atDef "" fields) (inField rules)
-      d = maybe "" (atDef "" fields) (outField rules)
+      i = maybe "" (atDef "" fields) (amountInField rules)
+      o = maybe "" (atDef "" fields) (amountOutField rules)
 
 tests_Hledger_Cli_Convert = TestList (test_parser ++ test_description_parsing)
 
