@@ -31,7 +31,6 @@ or ghci:
 >                  $-2  income
 >                   $1  liabilities
 > > l <- myLedger
-> > t <- myTimelog
 
 See "Hledger.Data.Ledger" for more examples.
 
@@ -47,6 +46,7 @@ import System.Exit
 import System.Process
 import Text.Printf
 
+import Hledger (ensureJournalFile)
 import Hledger.Cli.Add
 import Hledger.Cli.Balance
 import Hledger.Cli.Convert
@@ -69,10 +69,11 @@ main = do
   run' opts addons args
     where
       run' opts@CliOpts{command_=cmd} addons args
-       | "version" `in_` (rawopts_ opts)                 = putStrLn progversion
-       | "binary-filename" `in_` (rawopts_ opts)         = putStrLn $ binaryfilename progname
+       -- delicate, add tests before changing (eg --version, ADDONCMD --version, INTERNALCMD --version)
+       | (null matchedaddon) && "version" `in_` (rawopts_ opts)         = putStrLn progversion
+       | (null matchedaddon) && "binary-filename" `in_` (rawopts_ opts) = putStrLn $ binaryfilename progname
        | null cmd                                        = putStr $ showModeHelp mainmode'
-       | cmd `isPrefixOf` "add"                          = showModeHelpOr addmode      $ withJournalDo opts add
+       | cmd `isPrefixOf` "add"                          = showModeHelpOr addmode      $ journalFilePathFromOpts opts >>= ensureJournalFile >> withJournalDo opts add
        | cmd `isPrefixOf` "convert"                      = showModeHelpOr convertmode  $ convert opts
        | cmd `isPrefixOf` "test"                         = showModeHelpOr testmode     $ runtests opts
        | any (cmd `isPrefixOf`) ["accounts","balance"]   = showModeHelpOr accountsmode $ withJournalDo opts balance
@@ -80,14 +81,18 @@ main = do
        | any (cmd `isPrefixOf`) ["postings","register"]  = showModeHelpOr postingsmode $ withJournalDo opts register
        | any (cmd `isPrefixOf`) ["activity","histogram"] = showModeHelpOr activitymode $ withJournalDo opts histogram
        | cmd `isPrefixOf` "stats"                        = showModeHelpOr statsmode    $ withJournalDo opts stats
-       | not (null matchedaddon)                           = system shellcmd >>= exitWith
+       | not (null matchedaddon)                           = do
+                                                             when (debug_ opts) $ printf "running %s\n" shellcmd
+                                                             system shellcmd >>= exitWith
        | otherwise                                       = optserror ("command "++cmd++" is not recognized") >> exitFailure
        where
         mainmode' = mainmode addons
         showModeHelpOr mode f | "help" `in_` (rawopts_ opts) = putStr $ showModeHelp mode
                               | otherwise = f
-        matchedaddon = headDef "" $ filter (cmd `isPrefixOf`) addons
-        shellcmd = printf "%s-%s %s" progname matchedaddon (unwords' args)
+        matchedaddon | null cmd  = ""
+                     | otherwise = headDef "" $ filter (cmd `isPrefixOf`) addons
+        shellcmd = printf "%s-%s %s" progname matchedaddon (unwords' subcmdargs)
+        subcmdargs = args1 ++ drop 1 args2 where (args1,args2) = break (== cmd) $ filter (/="--") args
 
 {- tests:
 
