@@ -53,6 +53,7 @@ mainmode addons = defmode {
  ,modeGroupFlags = Group {
      groupUnnamed = helpflags
     ,groupHidden = [flagNone ["binary-filename"] (setboolopt "binary-filename") "show the download filename for this executable, and exit"]
+                   ++ fileflags -- quietly permit these flags before COMMAND as well
     ,groupNamed = []
     }
  ,modeArgs = ([], Just mainargsflag)
@@ -136,15 +137,9 @@ helpflags = [
  ,flagVersion (setboolopt "version")
  ]
 
-mainargsflag = flagArg f ""
-    where f s opts = let as = words' s
-                         cmd = headDef "" as
-                         args = drop (length cmd + 1) s
-                     in Right $ setopt "command" cmd $ setopt "args" args opts
-
+mainargsflag    = flagArg (\s opts -> Right $ setopt "args" s opts) ""
 commandargsflag = flagArg (\s opts -> Right $ setopt "args" s opts) "[PATTERNS]"
-
-addonargsflag = flagArg (\s opts -> Right $ setopt "args" s opts) "[ARGS]"
+addonargsflag   = flagArg (\s opts -> Right $ setopt "args" s opts) "[ARGS]"
 
 commandmode names = defmode {modeNames=names, modeValue=[("command",headDef "" names)]}
 
@@ -315,7 +310,7 @@ toCliOpts rawopts = do
                             ,quarterly_ = boolopt "quarterly" rawopts
                             ,yearly_    = boolopt "yearly" rawopts
                             ,format_    = maybestringopt "format" rawopts
-                            ,patterns_  = words'' prefixes $ singleQuoteIfNeeded $ stringopt "args" rawopts
+                            ,patterns_  = listofstringopt "args" rawopts
                             }
              }
 
@@ -323,7 +318,7 @@ toCliOpts rawopts = do
 getHledgerCliOpts :: [String] -> IO CliOpts
 getHledgerCliOpts addons = do
   args <- getArgs
-  toCliOpts (decodeRawOpts $ processValue (mainmode addons) $ moveFileOption args) >>= checkCliOpts
+  toCliOpts (decodeRawOpts $ processValue (mainmode addons) $ rearrangeForCmdArgs args) >>= checkCliOpts
 
 -- utils
 
@@ -349,12 +344,12 @@ getDirectoryContentsSafe d = getDirectoryContents d `catch` (\_ -> return [])
 -- | Convert possibly encoded option values to regular unicode strings.
 decodeRawOpts = map (\(name,val) -> (name, fromPlatformString val))
 
--- A workaround related to http://code.google.com/p/ndmitchell/issues/detail?id=457 :
--- we'd like to permit options before COMMAND as well as after it. Here we
--- make sure at least -f FILE will be accepted in either position.
-moveFileOption (fopt@('-':'f':_:_):cmd:rest) = cmd:fopt:rest
-moveFileOption ("-f":fval:cmd:rest) = cmd:"-f":fval:rest
-moveFileOption as = as
+-- A hacky workaround for http://code.google.com/p/ndmitchell/issues/detail?id=470 :
+-- we'd like to permit options before COMMAND as well as after it.
+-- Here we make sure at least -f FILE will be accepted in either position.
+rearrangeForCmdArgs (fopt@('-':'f':_:_):cmd:rest) = cmd:fopt:rest
+rearrangeForCmdArgs ("-f":fval:cmd:rest) = cmd:"-f":fval:rest
+rearrangeForCmdArgs as = as
 
 optserror = error' . (++ " (run with --help for usage)")
 
@@ -428,11 +423,15 @@ defaultBalanceFormatString = [
     , FormatField True Nothing Nothing Format.Account
     ]
 
--- | Get the journal file path from options, an environment variable, or a default
+-- | Get the journal file path from options, an environment variable, or a default.
+-- If the path contains a literal tilde raise an error to avoid confusion.
 journalFilePathFromOpts :: CliOpts -> IO String
 journalFilePathFromOpts opts = do
   f <- myJournalPath
-  return $ fromMaybe f $ file_ opts
+  return $ errorIfContainsTilde $ fromMaybe f $ file_ opts
+
+errorIfContainsTilde s |'~' `elem` s = error' "unsupported literal ~ found in environment variable, please adjust"
+                       | otherwise   = s
 
 aliasesFromOpts :: CliOpts -> [(AccountName,AccountName)]
 aliasesFromOpts = map parseAlias . alias_
